
#if WEB_STORE
#import "MFWebStoreReceipt.h"
#endif
#if APP_STORE
#import "MFAppStoreValidation.h"
#endif

#define STRINGIFY(a) _STRINGIFY(a)
#define _STRINGIFY(a) #a

// Use this key to detect usage of duplicate license by comparing between multiple copies of the app.
static inline __attribute__((always_inline)) NSString *MFDeduplicationKey(size_t *permittedCopies)
{
#ifdef CUSTOM_BRAND
	// reserve "+" prefix for custom brands followed by brand identifier
	if (permittedCopies) *permittedCopies = 0;
	return @"+" STRINGIFY(CUSTOM_BRAND);
#else
#if WEB_STORE
	if (MFWSReceiptLicenseValidate(@"dedup key"))
	{
		NSString *receiptDataString;
		MFWSReceipt receipt = MFWSReceiptCreateFromDefaults();
		MFWSLicenseInfo info = MFWSReceiptGetLicenseInfo(receipt);
		if (MFWSLicenseIsTrial(info)) {
			receiptDataString = @"T"; // web store trial license
			if (permittedCopies) *permittedCopies = 1;
		} else if (MFWSLicenseExpires(info)) {
			receiptDataString = @"X"; // web store expiring license
			if (permittedCopies) *permittedCopies = 0;
		} else {
			receiptDataString = @"P"; // web store permanent license with copy count
			if (permittedCopies) *permittedCopies = info.quantity;
			RHInteger value = RHReadValue((const char *)CFDataGetBytePtr(receipt.data), CFDataGetLength(receipt.data));
			receiptDataString = [receiptDataString stringByAppendingFormat:@"%llx", value];
		}
		MFWSReceiptRelease(&receipt);
		return receiptDataString;
	}
#endif
	if (MF_APP_STORE_CHECK1 && MF_APP_STORE_CHECK3 && MF_APP_STORE_CHECK2)
	{
		if (permittedCopies) *permittedCopies = 1;
		return @"A"; // can't deduplicate app store licenses
	}
#if APP_STORE
	if (permittedCopies) *permittedCopies = 1;
	return nil; // invalid app store receipt; will probably cash later
#else
	if (permittedCopies) *permittedCopies = 1;
	return @"T"; // local trial
#endif
#endif // else custom brand
}

